@@ -527,8 +527,12 @@ class Bookmarks {
 		$qb
 			->from('bookmarks', 'b')
 			->leftJoin('b', 'bookmarks_tags', 't', $qb->expr()->eq('t.bookmark_id', 'b.id'))
-			->leftJoin('b', 'bookmarks_folders_bookmarks', 'f', $qb->expr()->eq('f.bookmark_id', 'b.id'))
-			->groupBy(array_merge($selectedAttributes, [$sqlSortColumn]));
+			->leftJoin('b', 'bookmarks_folders_bookmarks', 'f', $qb->expr()->eq('f.bookmark_id', 'b.id'));
+		if (in_array('lastmodified', array_merge($selectedAttributes, [$sqlSortColumn]))) {
+			$qb->groupBy(['id', 'lastmodified']);
+		} else {
+			$qb->groupBy(['id']);
+		}
 
 		if ($userid !== -1) {
 			$qb->where($qb->expr()->eq('user_id', $qb->createPositionalParameter($userid)));
@@ -637,6 +641,26 @@ class Bookmarks {
 		$qb->having($filterExpression);
 	}
 
+	public function findBookmarksNeedingPreview($limit, $stalePeriod) {
+		$qb = $this->db->getQueryBuilder();
+		$qb->select('*');
+		$qb->from('bookmarks', 'b');
+		$qb->where($qb->expr()->lt('last_preview', $qb->createPositionalParameter(time()-$stalePeriod)));
+		$qb->orWhere($qb->expr()->isNull('last_preview'));
+		$qb->setMaxResults($limit);
+		$results = $qb->execute()->fetchAll();
+		return $results;
+	}
+
+	public function markPreviewCreated($id) {
+		$qb = $this->db->getQueryBuilder();
+		$qb
+			->update('bookmarks')
+			->set('last_preview', $qb->createNamedParameter(time()))
+			->where($qb->expr()->eq('id', $qb->createNamedParameter($id)));
+		return $qb->execute();
+	}
+
 	/**
 	 * @brief Delete bookmark with specific id
 	 * @param string $userId UserId
@@ -1126,40 +1150,50 @@ class Bookmarks {
 	 * @param string $file Content to import
 	 * @return null
 	 * */
-	public function importFile($userId, $file) {
+	public function importFile($userId, $file, $rootFolder=-1) {
+		$result = ['children' => [], 'errors' => []];
+		if (!$this->existsFolder($userId, $rootFolder)) {
+			$result['errors'][] = $this->l->t('Not allowed to access folder to import into');
+			return $result;
+		}
 		try {
 			$this->bookmarksParser->parse(file_get_contents($file), false);
 		} catch (\Exception $e) {
-			return [$e->message];
+			$result['errors'][] = $e->getMessage();
+			return $result;
 		}
-		$errors = [];
 		foreach ($this->bookmarksParser->currentFolder['children'] as $folder) {
-			$this->importFolder($userId, $folder, -1);
+			$rootContent['children'][] = $this->importFolder($userId, $folder, $rootFolder, $result['errors']);
 		}
 		foreach ($this->bookmarksParser->currentFolder['bookmarks'] as $bookmark) {
 			try {
-				$this->addBookmark($userId, $bookmark['href'], $bookmark['title'], $bookmark['tags'], $bookmark['description'], false, [-1]);
+				$bmId = $this->addBookmark($userId, $bookmark['href'], $bookmark['title'], $bookmark['tags'], $bookmark['description'], false, [$rootFolder]);
+				$rootContent['children'][] = ['type' => 'bookmark', 'id' => $bmId, 'title' => $bookmark['title'], 'url' => $bookmark['href']];
 			} catch (\InvalidArgumentException $e) {
 				$this->logger->logException($e, ['app' => 'bookmarks']);
-				$errors[] =  $this->l->t('Failed to import one bookmark, because: ') . $e->getMessage();
+				$result['errors'][] = $this->l->t('Failed to import one bookmark, because: ') . $e->getMessage();
 			}
 		}
-		return $errors;
+		return $rootContent;
 	}
 
-	private function importFolder($userId, $folder, $parentId) {
+	private function importFolder($userId, $folder, $parentId, &$errors = []) {
 		$folderId = $this->addFolder($userId, $folder['title'], $parentId);
+		$newFolder = ['type' => 'folder', 'id' => $folderId, 'title' => $folder['title'], 'children' => []];
 		foreach ($folder['bookmarks'] as $bookmark) {
 			try {
-				$this->addBookmark($userId, $bookmark['href'], $bookmark['title'], $bookmark['tags'], $bookmark['description'], false, [$folderId], $bookmark['add_date']->getTimestamp());
+				$add_date = isset($bookmark['add_date']) ? $bookmark['add_date']->getTimestamp() : null;
+				$bmId = $this->addBookmark($userId, $bookmark['href'], $bookmark['title'], $bookmark['tags'], $bookmark['description'], false, [$folderId], $add_date);
+				$newFolder['children'][] = ['type' => 'bookmark', 'id' => $bmId, 'title' => $bookmark['title'], 'url' => $bookmark['href']];
 			} catch (\InvalidArgumentException $e) {
 				$this->logger->logException($e, ['app' => 'bookmarks']);
 				$errors[] =  $this->l->t('Failed to import one bookmark, because: ') . $e->getMessage();
 			}
 		}
 		foreach ($folder['children'] as $childFolder) {
-			$this->importFolder($userId, $childFolder, $folderId);
+			$newFolder['children'][] = $this->importFolder($userId, $childFolder, $folderId, $errors);
 		}
+		return $newFolder;
 	}
 
 	/**
